module Jisota
  ##
  # Stores all information required to run a provision against one or multiple
  # servers (Except for the global package manager, which is stored in the Jisota module).
  #
  # Uses DSL methods in the initializer for convenient definition of packages, roles and servers
  class Configuration
    attr_accessor :packages, :roles, :servers, :ssh_engine

    def initialize(options = {}, &block)
      @packages   = options.fetch(:packages) { Collection.new }
      @roles      = options.fetch(:role) { Collection.new }
      @servers    = options.fetch(:servers) { [] }
      @ssh_engine = options.fetch(:ssh_engine) { SSHEngine.new }
      DSL.new(self).evaluate(&block) if block_given?
    end

    def add_package(package)
      packages.add(package)
    end

    def get_package(name)
      packages[name]
    end

    def add_role(role)
      roles.add(role)
    end

    def get_role(name)
      roles[name]
    end

    def add_server(server)
      servers << server
    end

    def each_server(&block)
      servers.each(&block)
    end

    class DSL < DSLBase
      def initialize(target)
        @target = target
      end

      def package(name, &block)
         @target.packages << Package.new(name, &block)
      end

      def role(name, &block)
        @target.roles << Role.new(name, &block)
      end

      def server(host, options = {})
        @target.servers << Server.new(host, options)
      end
    end
  end
end
