module Jisota
  ##
  # Responsible for outputting to the user / log file / whatever
  #
  # Will default to use STDOUT and STDERR, but that can be overwritten in the
  # initializer
  class Output
    attr_accessor :stdout, :stderr, :verbose, :prefix, :indent_level

    def initialize(options = {})
      @stdout  = options.fetch(:stdout) { $stdout }
      @stderr  = options.fetch(:stderr) { $stderr }
      @verbose = options.fetch(:verbose) { false }
      @prefix = options.fetch(:prefix) { "-----> " }
      @indent_level = 0
    end

    def system_message(message, &block)
      result = ["", prefix, "  " * indent_level, message, "\n"].join
      stdout.write(result)
      stdout.flush
      indent(&block) if block_given?
    end

    def info(message)
      stdout.write(message) if verbose
      stdout.flush
    end

    def warn(message)
      stderr.write(message) if verbose
      stderr.flush
    end

    def error(message)
      stderr.write(message)
      stderr.flush
    end

    def indent
      self.indent_level += 1
      if block_given?
        begin
          yield
        ensure
          outdent
        end
      end
    end

    def outdent
      self.indent_level -= 1
    end
  end
end
