module Jisota
  class Package
    attr_accessor :name, :description, :params, :sections

    def initialize(name = nil, &block)
      @params = []
      @sections = []
      @name = name
      DSL.new(self).evaluate(&block) if block_given?
    end

    alias_method :key, :name

    class InvalidBlockOrder < StandardError; end

    class Section
      attr_accessor :description, :run_block, :verify_block
    end

    class DSL < DSLBase
      def initialize(target)
        @target = target
      end

      def description(value)
        @target.description = value
      end

      def param(name, options = {})
        @target.params << Param.new(name, options)
      end

      def section(description)
        @target.sections << Section.new.tap do |section|
          section.description = description
        end
      end

      def run(&block)
        run_block = ScriptBlock.new(&block)
        section = @target.sections.last
        if section && section.run_block.nil?
          section.run_block = run_block
        else
          @target.sections << Section.new.tap do |new_section|
            new_section.run_block = run_block
          end
        end
      end

      def verify(&block)
        section = @target.sections.last
        if section.nil? || section.verify_block
          raise InvalidBlockOrder, "A verify block must come after a run block"
        end
        section.verify_block = ScriptBlock.new(&block)
      end
    end
  end
end
