module Jisota
  ##
  # DSL for creating a CompositeScript
  #
  # Methods in the DSL:
  #
  # [cmd] Add a CommandScript
  # [upload] Add a FileScript
  # [<arg name>]  The value of the argument
  # [<package name>] Add a PackageScript
  class ScriptBlock
    attr_accessor :block

    def initialize(options = {}, &block)
      @block = block
    end

    def evaluate(context = ScriptContext.new)
      CompositeScript.new.tap do |script|
        dsl = DSL.new(script, context)
        dsl.evaluate(&block)
      end
    end

    class DSL < DSLBase
      def initialize(script, context)
        @script = script
        @context = context
      end

      def cmd(command)
        @script.scripts << CommandScript.new(command)
      end

      def upload(**options, &block)
        @script.scripts << FileScript.new(**options, &block)
      end

      def method_missing(method, *args, &block)
        if has_argument?(method)
          get_argument(method)
        else
          add_package_script(method, args) || super
        end
      end

      private

      def has_argument?(name)
        @context.params.has_key?(name)
      end

      def get_argument(name)
        @context.params.fetch(name)
      end

      def add_package_script(name, args)
        if @context.packages.has_key?(name)
          package = @context.packages[name]
          @script.scripts << PackageScript.new(package, args)
        end
      end
    end
  end
end
